@@ -2,10 +2,10 @@
 package cloudrun
 
 import (
-	"github.com/hashicorp/waypoint-plugin-sdk"
+	sdk "github.com/hashicorp/waypoint-plugin-sdk"
 )
 
-//go:generate protoc -I ../../../.. --go_opt=plugins=grpc --go_out=../../../.. waypoint/builtin/google/cloudrun/plugin.proto
+//go:generate protoc -I ../../../.. -I ../../../thirdparty/proto/opaqueany --go_opt=plugins=grpc --go_out=../../../.. waypoint/builtin/google/cloudrun/plugin.proto
 
 // Options are the SDK options to use for instantiation for
 // the Google Cloud Run plugin.
