@@ -187,7 +187,7 @@ func (b *Builder) BuildODR(
 
 		err = ocis.SetupEntrypointLayer(refPath, data)
 		if err != nil {
-			return nil, status.Errorf(codes.Internal, "error setting up entrypoint layer: %s", err)
+			return nil, status.Errorf(codes.Internal, "error setting up entrypoint layer to host: %q, err: %s", ocis.Upstream, err)
 		}
 	}
 
