@@ -10,8 +10,16 @@ import (
 // from here but at the time of writing I was still building all the
 // mapper subsystems so I wanted to test it out.
 func PackImageMapper(src *DockerImage) *docker.Image {
-	return &docker.Image{
+	img := &docker.Image{
 		Image: src.Image,
 		Tag:   src.Tag,
 	}
+
+	if src.Remote {
+		img.Location = &docker.Image_Registry{Registry: &docker.Image_RegistryLocation{
+			WaypointGenerated: true,
+		}}
+	}
+
+	return img
 }
