@@ -536,6 +536,22 @@ func Commands(
 				baseCommand: baseCommand,
 			}, nil
 		},
+		"runner profile delete": func() (cli.Command, error) {
+			return &RunnerProfileDeleteCommand{
+				baseCommand: baseCommand,
+			}, nil
+		},
+
+		"runner install": func() (cli.Command, error) {
+			return &RunnerInstallCommand{
+				baseCommand: baseCommand,
+			}, nil
+		},
+		"runner uninstall": func() (cli.Command, error) {
+			return &RunnerUninstallCommand{
+				baseCommand: baseCommand,
+			}, nil
+		},
 
 		"context": func() (cli.Command, error) {
 			return &ContextHelpCommand{
@@ -695,6 +711,28 @@ func Commands(
 			}, nil
 		},
 
+		"task": func() (cli.Command, error) {
+			return &helpCommand{
+				SynopsisText: helpText["task"][0],
+				HelpText:     helpText["task"][1],
+			}, nil
+		},
+		"task list": func() (cli.Command, error) {
+			return &TaskListCommand{
+				baseCommand: baseCommand,
+			}, nil
+		},
+		"task inspect": func() (cli.Command, error) {
+			return &TaskInspectCommand{
+				baseCommand: baseCommand,
+			}, nil
+		},
+		"task cancel": func() (cli.Command, error) {
+			return &TaskCancelCommand{
+				baseCommand: baseCommand,
+			}, nil
+		},
+
 		"workspace": func() (cli.Command, error) {
 			return &helpCommand{
 				SynopsisText: helpText["workspace"][0],
@@ -1045,6 +1083,13 @@ This command contains further subcommands to work with servers.
 `,
 	},
 
+	"task": {
+		"Task introspection and management",
+		`
+Manage and check the status of On-Demand Runner Tasks in Waypoint.
+`,
+	},
+
 	"token": {
 		"Authenticate and invite collaborators",
 		`
