@@ -9,7 +9,6 @@ import (
 	"time"
 
 	"github.com/dustin/go-humanize"
-	"github.com/golang/protobuf/ptypes"
 	"github.com/posener/complete"
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
@@ -18,9 +17,8 @@ import (
 	clientpkg "github.com/hashicorp/waypoint/internal/client"
 	"github.com/hashicorp/waypoint/internal/clierrors"
 	"github.com/hashicorp/waypoint/internal/pkg/flag"
-	pb "github.com/hashicorp/waypoint/internal/server/gen"
-	serversort "github.com/hashicorp/waypoint/internal/server/sort"
 	"github.com/hashicorp/waypoint/internal/version"
+	pb "github.com/hashicorp/waypoint/pkg/server/gen"
 )
 
 type ReleaseListCommand struct {
@@ -92,7 +90,6 @@ func (c *ReleaseListCommand) Run(args []string) int {
 			c.project.UI.Output(clierrors.Humanize(err), terminal.WithErrorStyle())
 			return ErrSentinel
 		}
-		sort.Sort(serversort.ReleaseBundleCompleteDesc(resp.Releases))
 
 		if c.flagJson {
 			return c.displayJson(resp.Releases)
@@ -144,11 +141,11 @@ func (c *ReleaseListCommand) Run(args []string) int {
 
 			// Parse our times
 			var startTime, completeTime string
-			if t, err := ptypes.Timestamp(b.Status.StartTime); err == nil {
-				startTime = humanize.Time(t)
+			if b.Status.StartTime != nil {
+				startTime = humanize.Time(b.Status.StartTime.AsTime())
 			}
-			if t, err := ptypes.Timestamp(b.Status.CompleteTime); err == nil {
-				completeTime = humanize.Time(t)
+			if b.Status.CompleteTime != nil {
+				completeTime = humanize.Time(b.Status.CompleteTime.AsTime())
 			}
 
 			// Add status report information if we have any
@@ -168,7 +165,8 @@ func (c *ReleaseListCommand) Run(args []string) int {
 					statusReportComplete = "?"
 				}
 
-				if t, err := ptypes.Timestamp(statusReport.GeneratedTime); err == nil {
+				if statusReport.GeneratedTime != nil {
+					t := statusReport.GeneratedTime.AsTime()
 					statusReportComplete = fmt.Sprintf("%s - %s", statusReportComplete, humanize.Time(t))
 				}
 			}
