@@ -2,6 +2,7 @@ package cli
 
 import (
 	"bytes"
+	"context"
 	"crypto/ecdsa"
 	"crypto/elliptic"
 	"crypto/rand"
@@ -270,7 +271,8 @@ func (c *ServerRunCommand) Run(args []string) int {
 	}
 	if httpInsecureLn != nil {
 		values = append(values, terminal.NamedValue{
-			Name: "HTTP Address (Insecure)", Value: httpInsecureLn.Addr().String()})
+			Name: "HTTP Address (Insecure)", Value: httpInsecureLn.Addr().String(),
+		})
 	}
 	if auth {
 		values = append(values, terminal.NamedValue{Name: "Auth Required", Value: "yes"})
@@ -294,8 +296,8 @@ func (c *ServerRunCommand) Run(args []string) int {
 
 	// If we aren't bootstrapped, let the user know
 	if bs, ok := impl.(interface {
-		Bootstrapped() bool
-	}); ok && auth && !bs.Bootstrapped() {
+		Bootstrapped(ctx context.Context) bool
+	}); ok && auth && !bs.Bootstrapped(c.Ctx) {
 		c.ui.Output("Server requires bootstrapping!", terminal.WithHeaderStyle())
 		c.ui.Output("")
 		c.ui.Output(strings.TrimSpace(`
@@ -373,7 +375,9 @@ This command will bootstrap the server and setup a CLI context.
 			telemetryOptions = append(telemetryOptions, telemetry.WithDatadogExporter(
 				datadog.Options{
 					TraceAddr: c.flagTelemetryDatadogTraceAddr,
+					StatsAddr: c.flagTelemetryDatadogTraceAddr,
 					Service:   "waypoint",
+					Namespace: "waypoint",
 				},
 			))
 		}
