@@ -3,13 +3,12 @@ package cli
 import (
 	"strings"
 
-	"github.com/golang/protobuf/proto"
 	"github.com/posener/complete"
 
 	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
 	"github.com/hashicorp/waypoint/internal/clierrors"
 	"github.com/hashicorp/waypoint/internal/pkg/flag"
-	pb "github.com/hashicorp/waypoint/internal/server/gen"
+	pb "github.com/hashicorp/waypoint/pkg/server/gen"
 )
 
 type UserInspectCommand struct {
@@ -49,7 +48,7 @@ func (c *UserInspectCommand) Run(args []string) int {
 	user := userResp.User
 
 	// This isn't the most user friendly output but it gets the job done at the moment.
-	c.ui.Output(proto.MarshalTextString(user))
+	c.ui.Output(user.String())
 	return 0
 }
 
