@@ -17,8 +17,8 @@ import (
 	configpkg "github.com/hashicorp/waypoint/internal/config"
 	"github.com/hashicorp/waypoint/internal/pkg/finalcontext"
 	"github.com/hashicorp/waypoint/internal/pkg/gitdirty"
-	pb "github.com/hashicorp/waypoint/internal/server/gen"
-	"github.com/hashicorp/waypoint/internal/server/grpcmetadata"
+	pb "github.com/hashicorp/waypoint/pkg/server/gen"
+	"github.com/hashicorp/waypoint/pkg/server/grpcmetadata"
 )
 
 // job returns the basic job skeleton pre-populated with the correct
@@ -112,7 +112,7 @@ func (c *Project) setupLocalJobSystem(ctx context.Context) (isLocal bool, newCtx
 	if *c.useLocalRunner {
 		if c.activeRunner == nil {
 			// we need a local runner and we haven't started it yet
-			if err := c.startRunner(); err != nil {
+			if err := c.startRunner(ctx); err != nil {
 				return false, newCtx, errors.Wrapf(err, "failed to start local runner for job %s", err)
 			}
 		}
@@ -127,7 +127,6 @@ func (c *Project) setupLocalJobSystem(ctx context.Context) (isLocal bool, newCtx
 		gitDirtyErr := func() error {
 			// Running this inside of an anonymous func so that we can
 			// return early
-
 			if c.configPath == "" {
 				// No local project dir, so nothing is dirty!
 				return nil
@@ -234,7 +233,7 @@ func (c *Project) doJobMonitored(ctx context.Context, job *pb.Job, ui terminal.U
 				},
 			},
 		}
-	} else {
+	} else if c.waypointHCL != nil {
 		var configRunner *configpkg.Runner
 		// Find runner configuration on the app
 		if job.Application != nil {
@@ -246,8 +245,10 @@ func (c *Project) doJobMonitored(ctx context.Context, job *pb.Job, ui terminal.U
 		}
 		// If runner config is found, assign to job
 		if configRunner != nil {
-			job.OndemandRunner = &pb.Ref_OnDemandRunnerConfig{
-				Name: configRunner.Profile,
+			if configRunner.Profile != "" {
+				job.OndemandRunner = &pb.Ref_OnDemandRunnerConfig{
+					Name: configRunner.Profile,
+				}
 			}
 		}
 	}
@@ -446,10 +447,6 @@ func (c *Project) queueAndStreamJob(
 
 					ui.Table(tbl)
 				case *pb.GetJobStreamResponse_Terminal_Event_StepGroup_:
-					if sg != nil {
-						sg.Wait()
-					}
-
 					if !ev.StepGroup.Close {
 						sg = ui.StepGroup()
 					}
@@ -541,7 +538,8 @@ func (c *Project) queueAndStreamJob(
 			switch event.State.Current {
 			case pb.Job_QUEUED:
 				stateEventTimer = time.AfterFunc(stateEventPause, func() {
-					ui.Output("Operation is queued. Waiting for runner assignment...",
+					ui.Output("Operation is queued waiting for job %q. Waiting for runner assignment...",
+						queueResp.JobId,
 						terminal.WithHeaderStyle())
 					ui.Output("If you interrupt this command, the job will still run in the background.",
 						terminal.WithInfoStyle())
