@@ -14,15 +14,14 @@ import (
 	empty "google.golang.org/protobuf/types/known/emptypb"
 	"google.golang.org/protobuf/types/known/timestamppb"
 
+	"github.com/hashicorp/waypoint-plugin-sdk/terminal"
 	"github.com/hashicorp/waypoint/pkg/server"
 	pb "github.com/hashicorp/waypoint/pkg/server/gen"
-	"github.com/hashicorp/waypoint/pkg/server/logbuffer"
 	serverptypes "github.com/hashicorp/waypoint/pkg/server/ptypes"
 	"github.com/hashicorp/waypoint/pkg/serverconfig"
 	"github.com/hashicorp/waypoint/pkg/serverstate"
 )
 
-// TODO: test
 func (s *Service) GetJob(
 	ctx context.Context,
 	req *pb.GetJobRequest,
@@ -38,7 +37,6 @@ func (s *Service) GetJob(
 	return job.Job, nil
 }
 
-// TODO: test
 func (s *Service) ListJobs(
 	ctx context.Context,
 	req *pb.ListJobsRequest,
@@ -72,25 +70,27 @@ func (s *Service) queueJobMulti(
 	ctx context.Context,
 	req []*pb.QueueJobRequest,
 ) ([]*pb.QueueJobResponse, error) {
-	jobs := make([]*pb.Job, 0, len(req))
+	jobQueue := make([]*pb.Job, 0, len(req)*4)
+	jobIds := make([]string, 0, len(req))
 	for _, single := range req {
-		job, _, err := s.queueJobReqToJob(ctx, single)
+		jobs, jobId, err := s.queueJobReqToJob(ctx, single)
 		if err != nil {
 			return nil, err
 		}
 
-		jobs = append(jobs, job...)
+		jobQueue = append(jobQueue, jobs...)
+		jobIds = append(jobIds, jobId)
 	}
 
 	// Queue the jobs
-	if err := s.state(ctx).JobCreate(jobs...); err != nil {
+	if err := s.state(ctx).JobCreate(jobQueue...); err != nil {
 		return nil, err
 	}
 
 	// Get the response
-	resp := make([]*pb.QueueJobResponse, len(jobs))
-	for i, job := range jobs {
-		resp[i] = &pb.QueueJobResponse{JobId: job.Id}
+	resp := make([]*pb.QueueJobResponse, len(jobIds))
+	for i, id := range jobIds {
+		resp[i] = &pb.QueueJobResponse{JobId: id}
 	}
 
 	return resp, nil
@@ -221,6 +221,35 @@ func (s *Service) QueueJob(
 // wrapJobWithRunner takes a job and "wraps" it within an on-demand launched
 // runner. This creates a dependency chain that ensures that the runner is
 // started and stopped around the given job (hence "wraps").
+//
+// A diagram of the dependency chain created is shown below. The dashed
+// border is the "source" job.
+//
+//           ┌────────────────┐
+//           │   Start Task   │─────────────┐
+//           └────────────────┘             │
+//                    │                     │
+//          ┌─────────┴─────────┐           │
+//          ▼                   ▼           │
+// ┌────────────────┐   ┌─ ── ── ── ── ──   │
+// │   Watch Task   │   │      Job       │  │
+// └────────────────┘   └ ── ── ── ── ── ┘  │
+//          │                    │          │
+//          └─────────┬──────────┘          │
+//                    ▼                     │
+//           ┌────────────────┐             │
+//           │   Stop Task    │◀────────────┘
+//           └────────────────┘
+//
+// Details:
+//
+//   - Start task launches the on-demand runner.
+//   - After it is launched, "job" can run targeting the launched ODR.
+//   - Simultaneously, the watch task watches the launched task and records
+//     logs, exit code, etc.
+//   - Finally, stop task is called to clean up the resources associated
+//     with start.
+//
 func (s *Service) wrapJobWithRunner(
 	ctx context.Context,
 	source *pb.Job,
@@ -242,6 +271,12 @@ func (s *Service) wrapJobWithRunner(
 		return nil, err
 	}
 
+	// Generate our job to watch the ODR
+	watchJob, err := s.onDemandRunnerWatchJob(ctx, startJob, source, od)
+	if err != nil {
+		return nil, err
+	}
+
 	// Change our source job to run on the launched ODR.
 	source.TargetRunner = &pb.Ref_Runner{
 		Target: &pb.Ref_Runner_Id{
@@ -255,21 +290,57 @@ func (s *Service) wrapJobWithRunner(
 	source.DependsOn = []string{startJob.Id}
 
 	// Job to stop the ODR
-	stopJob, err := s.onDemandRunnerStopJob(ctx, startJob, source, od)
+	stopJob, err := s.onDemandRunnerStopJob(ctx, startJob, watchJob, source, od)
 	if err != nil {
 		return nil, err
 	}
 
+	// Write a Task state with the On-Demand Runner job triple
+	task := &pb.Task{
+		StartJob: &pb.Ref_Job{Id: startJob.Id},
+		TaskJob:  &pb.Ref_Job{Id: source.Id},
+		StopJob:  &pb.Ref_Job{Id: stopJob.Id},
+		WatchJob: &pb.Ref_Job{Id: watchJob.Id},
+		JobState: pb.Task_PENDING,
+	}
+	if err := s.state(ctx).TaskPut(task); err != nil {
+		return nil, err
+	} else {
+		task, err := s.state(ctx).TaskGet(&pb.Ref_Task{
+			Ref: &pb.Ref_Task_JobId{
+				JobId: source.Id,
+			},
+		})
+		if err != nil {
+			// could not find task that was just Put into the db!
+			return nil, err
+		}
+
+		// assign a task ref to each job for lookup later
+		taskRef := &pb.Ref_Task{
+			Ref: &pb.Ref_Task_Id{
+				Id: task.Id,
+			},
+		}
+
+		startJob.Task = taskRef
+		source.Task = taskRef
+		stopJob.Task = taskRef
+		watchJob.Task = taskRef
+	}
+
 	// These must be in order of dependency currently. This is a limitation
 	// of the state.JobCreate API and we should fix it one day. If we get
 	// this wrong it'll just error, so we'll know quickly.
 	return []*pb.Job{
 		startJob,
+		watchJob,
 		source,
 		stopJob,
 	}, nil
 }
 
+// onDemandRunnerStartJob generates a StartJob template for a Task.
 func (s *Service) onDemandRunnerStartJob(
 	ctx context.Context,
 	source *pb.Job,
@@ -315,8 +386,9 @@ func (s *Service) onDemandRunnerStartJob(
 
 	encodedDefaultUserId, err := s.encodeId(ctx, DefaultUserId)
 	if err != nil {
-		log.Error("failed to encode the default user id", "id", DefaultUserId, "err", err)
-		return nil, "", status.Error(codes.Internal, "failed to encode the default user id")
+		msg := "failed to encode the default user id when starting and ODR job"
+		log.Error(msg, "id", DefaultUserId, "err", err)
+		return nil, "", status.Error(codes.Internal, msg)
 	}
 
 	// We generate a new login token for each ondemand-runner used. This will inherit
@@ -395,7 +467,8 @@ func (s *Service) onDemandRunnerStartJob(
 	return job, runnerId, nil
 }
 
-func (s *Service) onDemandRunnerStopJob(
+// onDemandRunnerWatchJob generates a WatchJob template for a Task.
+func (s *Service) onDemandRunnerWatchJob(
 	ctx context.Context,
 	startJob *pb.Job,
 	source *pb.Job,
@@ -406,10 +479,48 @@ func (s *Service) onDemandRunnerStopJob(
 		Workspace:   source.Workspace,
 		Application: source.Application,
 
+		// We depend on the starting job. We don't run if the start job fails.
+		DependsOn: []string{startJob.Id},
+
+		// Use the same targeting as the start job. We assume the start job
+		// had proper access to stop, too, so we just copy it.
+		TargetRunner: startJob.TargetRunner,
+
+		// Watch
+		Operation: &pb.Job_WatchTask{
+			WatchTask: &pb.Job_WatchTaskOp{
+				StartJob: &pb.Ref_Job{Id: startJob.Id},
+			},
+		},
+	}
+
+	// Get the next id for the job
+	id, err := server.Id()
+	if err != nil {
+		return nil, status.Errorf(codes.Internal, "uuid generation failed: %s", err)
+	}
+	job.Id = id
+
+	return job, nil
+}
+
+// onDemandRunnerStopJob generates a StopJob template for a Task.
+func (s *Service) onDemandRunnerStopJob(
+	ctx context.Context,
+	startJob *pb.Job,
+	watchJob *pb.Job,
+	source *pb.Job,
+	od *pb.OnDemandRunnerConfig,
+) (*pb.Job, error) {
+	job := &pb.Job{
+		// Inherit the workspace/application of the source job.
+		Workspace:   source.Workspace,
+		Application: source.Application,
+
 		// We depend on both the start job and the main job. We allow them
 		// both to fail, however, because we want to try to stop no matter what.
-		DependsOn:             []string{startJob.Id, source.Id},
-		DependsOnAllowFailure: []string{startJob.Id, source.Id},
+		DependsOn:             []string{startJob.Id, watchJob.Id, source.Id},
+		DependsOnAllowFailure: []string{startJob.Id, watchJob.Id, source.Id},
 
 		// Use the same targeting as the start job. We assume the start job
 		// had proper access to stop, too, so we just copy it.
@@ -517,6 +628,7 @@ func (s *Service) GetJobStream(
 			ws = memdb.NewWatchSet()
 			job, err = s.state(ctx).JobById(job.Id, ws)
 			if err != nil {
+				log.Error("error acquiring job by id", "error", err, "id", req.JobId)
 				errCh <- err
 				return
 			}
@@ -603,17 +715,52 @@ func (s *Service) GetJobStream(
 				}
 			}
 
-			// If we haven't initialized output streaming and the output buffer
-			// is now non-nil, initialize that. This will send any buffered
-			// data down.
-			if eventsCh == nil && job.OutputBuffer != nil {
-				eventsCh, err = s.getJobStreamOutputInit(ctx, job, server)
-				if err != nil {
-					return err
+			if eventsCh == nil {
+				switch job.State {
+				case pb.Job_RUNNING:
+
+					// We're seeing the job start up live, so we'll initialize the
+					// event channel and use the job streamer to stream the logs
+					// in as they arrive via the runner interface.
+					// If the job OutputBuffer is nil, the streamer won't send
+					// any events.
+					eventsCh, err = s.getJobStreamOutputInit(ctx, log, job, server)
+					if err != nil {
+						msg := "failed to init job output stream"
+						log.Error(msg, "job.id", job.Id, "error", err)
+						return status.Error(codes.Internal, msg)
+					}
+
+				// NOTE: at present (2022-04-20) the CLI does not utilize this
+				// code path but the UI does for historical job viewing.
+				case pb.Job_SUCCESS, pb.Job_ERROR:
+					// This means that the requested stream finished before GetJobStream was
+					// called. As such, we'll replay the output events.
+					events, err := s.logStreamProvider.ReadCompleted(ctx, log, s.state(ctx), job)
+					if err != nil {
+						msg := "failed to stream logs for completed job"
+						log.Error(msg, "job.Id", job.Id, "error", err)
+						return status.Error(codes.Internal, msg)
+					}
+
+					// We're doing this synchronously so that the client receives the events
+					// before we send down the completion event.
+					if err := server.Send(&pb.GetJobStreamResponse{
+						Event: &pb.GetJobStreamResponse_Terminal_{
+							Terminal: &pb.GetJobStreamResponse_Terminal{
+								Events:   events,
+								Buffered: true,
+							},
+						},
+					}); err != nil {
+						log.Error("failed to send logs for completed job", "job.Id", job.Id, "error", err)
+						return err
+					}
 				}
 			}
 
 			switch job.State {
+
 			case pb.Job_SUCCESS, pb.Job_ERROR:
 				// TODO(mitchellh): we should drain the output buffer
 
@@ -643,30 +790,24 @@ func (s *Service) GetJobStream(
 	}
 }
 
-func (s *Service) readJobLogBatch(r *logbuffer.Reader, block bool) []*pb.GetJobStreamResponse_Terminal_Event {
-	entries := r.Read(64, block)
-	if entries == nil {
-		return nil
-	}
-
-	events := make([]*pb.GetJobStreamResponse_Terminal_Event, len(entries))
-	for i, entry := range entries {
-		events[i] = entry.(*pb.GetJobStreamResponse_Terminal_Event)
-	}
-
-	return events
-}
-
 func (s *Service) getJobStreamOutputInit(
 	ctx context.Context,
+	log hclog.Logger,
 	job *serverstate.Job,
 	server pb.Waypoint_GetJobStreamServer,
 ) (<-chan []*pb.GetJobStreamResponse_Terminal_Event, error) {
+	// Start a log stream reader for this job
+	lsReader, err := s.logStreamProvider.StartReader(ctx, log, job)
+	if err != nil {
+		return nil, errors.Wrapf(err, "failed to start log reader")
+	}
+
 	// Send down all our buffered lines.
-	outputR := job.OutputBuffer.Reader(-1)
-	go outputR.CloseContext(ctx)
 	for {
-		events := s.readJobLogBatch(outputR, false)
+		events, err := lsReader.ReadStream(ctx, false)
+		if err != nil {
+			return nil, errors.Wrapf(err, "failed to read buffered log batch")
+		}
 		if events == nil {
 			break
 		}
@@ -683,11 +824,47 @@ func (s *Service) getJobStreamOutputInit(
 		}
 	}
 
-	// Start a goroutine that reads output
+	// Start a goroutine that reads output. If things go wrong in here,
+	// we cancel the whole job stream request.
+	ctx, cancel := context.WithCancel(ctx)
 	eventsCh := make(chan []*pb.GetJobStreamResponse_Terminal_Event, 1)
 	go func() {
 		for {
-			events := s.readJobLogBatch(outputR, true)
+			events, err := lsReader.ReadStream(ctx, true)
+			if err != nil {
+				// In the event of a reader error, we shut the stream down.
+				// It's up to the reader to retry if ephemeral errors are common.
+
+				msg := "failed to read streaming log batch"
+				log.Error(msg, "error", err)
+
+				// Let the client know we're terminating due to an error.
+				if err := server.Send(&pb.GetJobStreamResponse{
+					Event: &pb.GetJobStreamResponse_Terminal_{
+						Terminal: &pb.GetJobStreamResponse_Terminal{
+							Events: []*pb.GetJobStreamResponse_Terminal_Event{{
+								Timestamp: timestamppb.Now(),
+
+								// NOTE(izaak): really not sure if this is the right kind of terminal event
+								Event: &pb.GetJobStreamResponse_Terminal_Event_Line_{
+									Line: &pb.GetJobStreamResponse_Terminal_Event_Line{
+										Style: terminal.ErrorStyle,
+										Msg:   msg,
+									},
+								},
+							}},
+							Buffered: false,
+						},
+					},
+				}); err != nil {
+					// This waypoint server must be experiencing some kind of catastrophic failure - it can't
+					// stream logs or send messages to the client.
+					log.Error("failed to inform client of read streaming error", "error", err)
+				}
+
+				cancel()
+				return
+			}
 			if events == nil {
 				return
 			}
