@@ -4,6 +4,7 @@ import (
 	"context"
 	"io"
 	"strings"
+	"time"
 
 	"github.com/hashicorp/go-hclog"
 	"github.com/hashicorp/go-memdb"
@@ -14,6 +15,7 @@ import (
 	"google.golang.org/protobuf/types/known/emptypb"
 	empty "google.golang.org/protobuf/types/known/emptypb"
 
+	"github.com/hashicorp/waypoint/internal/telemetry/metrics"
 	pb "github.com/hashicorp/waypoint/pkg/server/gen"
 	"github.com/hashicorp/waypoint/pkg/server/logstream"
 	serverptypes "github.com/hashicorp/waypoint/pkg/server/ptypes"
@@ -536,6 +538,12 @@ func (s *Service) RunnerJobStream(
 	log.Trace("loaded config sources for job", "total_sourcers", len(cfgSrcs))
 
 	log.Debug("sending job assignment to runner")
+
+	operation := operationString(job.Job)
+	defer func(start time.Time) {
+		metrics.MeasureOperation(ctx, start, operation)
+	}(time.Now())
+	metrics.CountOperation(ctx, operation)
 	// Send the job assignment.
 	//
 	// If this has an error, we continue to accumulate the error until
@@ -615,15 +623,15 @@ func (s *Service) RunnerJobStream(
 		return err
 	}
 
-	var logStreamTracker logstream.Tracker
+	var logStreamWriter logstream.Writer
 	if s.logStreamProvider != nil {
-		logStreamTracker, err = s.logStreamProvider.StartWriter(ctx, log, s.state(ctx), job)
+		logStreamWriter, err = s.logStreamProvider.StartWriter(ctx, log, s.state(ctx), job)
 		if err != nil {
 			return errors.Wrapf(err, "failed to start a log writer to handle jog logs")
 		}
 	}
 
-	defer logStreamTracker.Flush(ctx)
+	defer logStreamWriter.Flush(ctx)
 
 	// Start a goroutine that watches for job changes
 	jobCh := make(chan *serverstate.Job, 1)
@@ -712,7 +720,7 @@ func (s *Service) RunnerJobStream(
 			for {
 				select {
 				case req := <-eventCh:
-					if err := s.handleJobStreamRequest(log, job, server, req, logStreamTracker); err != nil {
+					if err := s.handleJobStreamRequest(log, job, server, req, logStreamWriter); err != nil {
 						return err
 					}
 				default:
@@ -724,7 +732,7 @@ func (s *Service) RunnerJobStream(
 			return err
 
 		case req := <-eventCh:
-			if err := s.handleJobStreamRequest(log, job, server, req, logStreamTracker); err != nil {
+			if err := s.handleJobStreamRequest(log, job, server, req, logStreamWriter); err != nil {
 				return err
 			}
 
@@ -770,7 +778,7 @@ func (s *Service) handleJobStreamRequest(
 	job *serverstate.Job,
 	srv pb.Waypoint_RunnerJobStreamServer,
 	req *pb.RunnerJobStreamRequest,
-	logStreamTracker logstream.Tracker,
+	logStreamWriter logstream.Writer,
 ) error {
 	ctx := srv.Context()
 	log.Trace("event received", "event", req.Event)
@@ -806,9 +814,8 @@ func (s *Service) handleJobStreamRequest(
 		})
 
 	case *pb.RunnerJobStreamRequest_Terminal:
-
 		// Write the events
-		logStreamTracker.NewEvent(ctx, event)
+		logStreamWriter.NewEvent(ctx, event)
 
 		return nil
 
@@ -875,3 +882,50 @@ func (s *Service) runnerVerifyToken(
 
 	return nil
 }
+
+func operationString(job *pb.Job) string {
+	// Types that are assignable to Operation:
+	switch job.Operation.(type) {
+	case *pb.Job_Noop_:
+		return "noop"
+	case *pb.Job_Build:
+		return "build"
+	case *pb.Job_Push:
+		return "push"
+	case *pb.Job_Deploy:
+		return "deploy"
+	case *pb.Job_Destroy:
+		return "destroy"
+	case *pb.Job_Release:
+		return "release"
+	case *pb.Job_Validate:
+		return "validate"
+	case *pb.Job_Auth:
+		return "auth"
+	case *pb.Job_Docs:
+		return "docs"
+	case *pb.Job_ConfigSync:
+		return "config_sync"
+	case *pb.Job_Exec:
+		return "exec"
+	case *pb.Job_Up:
+		return "up"
+	case *pb.Job_Logs:
+		return "logs"
+	case *pb.Job_QueueProject:
+		return "queue_project"
+	case *pb.Job_Poll:
+		return "poll"
+	case *pb.Job_StatusReport:
+		return "status_report"
+	case *pb.Job_StartTask:
+		return "start_task"
+	case *pb.Job_StopTask:
+		return "stop_task"
+	case *pb.Job_WatchTask:
+		return "watch_task"
+	case *pb.Job_Init:
+		return "init"
+	}
+	return "unknown"
+}
