@@ -1,10 +1,7 @@
-import { Model, hasMany } from 'ember-cli-mirage';
+import { Model } from 'ember-cli-mirage';
 import { Ref, Workspace } from 'waypoint-pb';
 
 export default Model.extend({
-  builds: hasMany(),
-  statusReports: hasMany(),
-
   toProtobuf(): Workspace {
     let result = new Workspace();
 
